import java.util.*;
import java.io.*;

/** An object of this class describes the rules for a particular variation of
 * movable tic-tac-toe, specifically, the moves that are possible from
 * a given state, and whether a given state is a win for one of the
 * players.
 */
public class GameRules {

    private boolean moveMatrix[][][][];

    /** Constructs a <tt>GameRules</tt> object in which legal moves are defined
     * by the given array of 4-tuples.  Each 4-tuple is of the form
     * <tt>(fx, fy, tx, ty)</tt> with the meaning that it is legal to move a
     * piece from position <tt>fx, fy</tt> to position <tt>tx, ty</tt>.
     */
    public GameRules(int[][] ml) {
	moveMatrix = new boolean[3][3][3][3];

	for (int i = 0; i < ml.length; i++) {
	    moveMatrix[ml[i][0]][ml[i][1]][ml[i][2]][ml[i][3]] = true;
	}
    }

    /** Constructs a <tt>GameRules</tt> object for a game whose legal
     * moves are stored in the file <tt>game_id.rules</tt>.
     */
    public GameRules(String game_id) throws IllegalArgumentException, IOException {
	moveMatrix = new boolean[3][3][3][3];

	BufferedReader in;
	String line;
	
	try {
	    in = new BufferedReader(new FileReader(game_id + ".rules"));
	}
	catch (FileNotFoundException e) {
	    System.err.print("File "+game_id+".rules not found.\n");
	    throw e;
	}
	try {
	    while ((line = in.readLine()) != null) {
		line = line.trim();
		if (line.length() == 0) continue;
		moveMatrix
		    [Character.getNumericValue(line.charAt(0))]
		    [Character.getNumericValue(line.charAt(1))]
		    [Character.getNumericValue(line.charAt(2))]
		    [Character.getNumericValue(line.charAt(3))]
		    = true;
	    }
	}
	catch (IOException e) {
	    System.err.print("Error reading file "+game_id+".rules.\n");
	    throw e;
	}
	catch (IllegalArgumentException e) {
	    System.err.print("File "+game_id+".rules has wrong format.\n");
	    throw e;
	}
	try {
	    in.close();
	}
	catch (IOException e) {
	    System.err.print("Error closing file "+game_id+".rules.\n");
	    throw e;
	}
    }

    /** Returns an array of all possible legal moves from the given game state.
     * @param gs given game state
     */
    public Move[] legalMoves(GameState gs) {
	LinkedList moveList = new LinkedList();

	int numPlayed = 0;
	for (int fx = 0; fx < 3; fx++) for (int fy = 0; fy < 3; fy++) {
	    if (gs.square(fx,fy) != 0)
		numPlayed++;
	}

	if (numPlayed < 6)
	    for (int tx = 0; tx < 3; tx++) for (int ty = 0; ty < 3; ty++) {
		if (gs.square(tx,ty) == 0)
		    moveList.addFirst(new Move(tx,ty));
	    }
	else {
	    for (int fx = 0; fx < 3; fx++) for (int fy = 0; fy < 3; fy++) {
		if (gs.square(fx,fy) != gs.turn())
		    continue;
		for (int tx = 0; tx < 3; tx++) for (int ty = 0; ty < 3; ty++) {
		    if (moveMatrix[fx][fy][tx][ty] && gs.square(tx,ty) == 0)
			moveList.addFirst(new Move(fx,fy,tx,ty));
		}
	    }
	}

	int s = moveList.size();

	Move[] moves = new Move[s];
	for (int i = 0; i < s; i++) {
	    moves[i] = (Move) moveList.removeFirst();
	}
	return moves;
    }

    /** Returns +1 if player +1 has won in the given game state, 
     * -1 if player -1 has won, and 0 if no one has won.
     * @param gs game state to be checked
     */
    public int isWin(GameState gs) {
	int v;

	for (int x = 0; x < 3; x++) {
	    if ((v = gs.square(x,0)) != 0 &&
		(v == gs.square(x,1)) &&
		(v == gs.square(x,2)))
		return v;
	}
	for (int y = 0; y < 3; y++) {
	    if ((v = gs.square(0,y)) != 0 &&
		(v == gs.square(1,y)) &&
		(v == gs.square(2,y)))
		return v;
	}

	if ((v = gs.square(1,1)) != 0
	    && (   (v == gs.square(0,0) && v == gs.square(2,2)) 
                || (v == gs.square(2,0) && v == gs.square(0,2)) )
	    )
	    return v;
	return 0;
    }

    /** Tests whether the given game state and the move from it are legal
     * according to the game rules.
     * @param gs game state to be checked
     * @param move move to be checked
     */
    public boolean isLegal(GameState gs, Move move) {
	Move[] move_list = legalMoves(gs);

	for (int i = 0; i < move_list.length; i++) {
	    if (move.equals(move_list[i]))
		return true;
	}
	return false;
    }
}
