import java.util.*;

/** An object of this class represents the state of the game,
 * including the current board position (accessible using the
 * <tt>square</tt> method) and whose turn it is (accessible using the
 * <tt>turn</tt> method).  A new game position can be generated using
 * the <tt>applyMove</tt> method. Squares are indexed by pairs
 * <tt>x,y</tt> where each is 0, 1 or 2.  Players are encoded as +1 or
 * -1, with +1 moving first.
 */
public class GameState {
    private int squares[][];
    private int turn;

    /** Constructs a new game state with no pieces on the board, and
     * +1's turn.
     */
    public GameState() {
	squares = new int[3][3];
	for (int x = 0; x < 3; x++) for (int y = 0; y < 3; y++)
	    squares[x][y] = 0;
	turn = 1;
    }

    /** Constructs a new game state with pieces on board as specified
     * by <tt>squares</tt>; the player whose turn it is is specified by
     * <tt>turn</tt>.
     * @param squares <tt>squares[x][y]</tt> specifies the contents of
     * the square <tt>x,y</tt>.  +1 or -1 indicates that one of the
     * players has a piece in this square.  0 indicates that the
     * square is empty.
     */
    public GameState(int[][] squares, int turn) {
	this.squares = new int[3][3];
	for (int x = 0; x < 3; x++) for (int y = 0; y < 3; y++)
	    this.squares[x][y] = squares[x][y];
	this.turn = turn;
    }

    /** Returns the contents of square <tt>x,y</tt> on the
     * current board position.
     */
    public int square(int x, int y) {
	return squares[x][y];
    }

    /** Returns the player (+1 or -1) whose turn it is. */
    public int turn() {
	return turn;
    }

    /** Constructs and returns a new game state obtained from this
     * game state by applying the given <tt>Move</tt> object to this
     * <tt>GameState</tt>.  No error checking is done to be sure that
     * such a move is legal or possible.
     * @param m <tt>Move</tt> to be applied
     * @return new game state
     */
    public GameState applyMove(Move m) {
	GameState new_gs = new GameState();
	for (int x = 0; x < 3; x++) for (int y = 0; y < 3; y++)
	    new_gs.squares[x][y] = this.squares[x][y];
	new_gs.squares[m.tx][m.ty] = this.turn;
	if (m.fx >= 0) {
	    new_gs.squares[m.fx][m.fy] = 0;
	}
	new_gs.turn = -this.turn;
	return new_gs;
    }

    /** Returns the hash code for this game position.  This method is
     * provided so that <tt>GameState</tt> objects can be reliably
     * inserted into hash tables and other data structures provided
     * with the Standard Java Platform.
     */
    public int hashCode() {
	int h = 0;
	for (int x = 0; x < 3; x++) for (int y = 0; y < 3; y++)
	    h = h*3 + squares[x][y] + 1;
	h = h*3 + turn + 1;
	return h;
    }

    /** Compares the specified object to this game state for equality.
     * Returns <tt>true</tt> if the object is also a game state, and
     * if it represents an identical board position, and it is the
     * same player's turn.
     * @param o object to be compared
     */
    public boolean equals(Object o) {
	GameState gs;
	try {
	    gs = (GameState) o;
	}
	catch (ClassCastException e) {
	    return false;
	}
	for (int x = 0; x < 3; x++) for (int y = 0; y < 3; y++)
	    if (gs.squares[x][y] != this.squares[x][y])
		return false;
	if (gs.turn != this.turn)
	    return false;
	return true;
    }

    private String toString(int x, int y) {
	return (squares[x][y] < 0 ? "-" : squares[x][y] > 0 ? "+" : ".");
    }

    /** Produces a one line string description of the GameState object.
     */
    public String toString() {
	return toString(0,2) + toString(1,2) + toString(2,2) + " "
	    +  toString(0,1) + toString(1,1) + toString(2,1) + " "
	    +  toString(0,0) + toString(1,0) + toString(2,0) + " "
	    +  (turn>0 ? "+" : "-");
    }

    /** Produces a multi-line string description of the GameState
     *  object indented <tt>indentBy</tt> spaces.
     */
    public String toLongString(int indentBy) {
	String indent = "";
	for (int i=0; i<indentBy; i++)
	    indent = indent + " ";
	return
	      indent + " =======\n"
	    + indent + "2|" + toString(0,2) + " " + toString(1,2) + " " + toString(2,2) + "|\n"
	    + indent + "1|" + toString(0,1) + " " + toString(1,1) + " " + toString(2,1) + "|  "
	    + (turn>0 ? "+" : "-") + "\n"
	    + indent + "0|" + toString(0,0) + " " + toString(1,0) + " " + toString(2,0) + "|\n"
	    + indent + " =======\n"
	    + indent + "  0 1 2\n";
    }
}
