import java.util.*;
import java.io.*;

public class Referee {
    private Player player_pos, player_neg;
    private int maxMoves;
    private GameRules gr;

    public Referee(Player player_pos, Player player_neg, GameRules gr, int maxMoves) {
	this.player_pos = player_pos;
	this.player_neg = player_neg;
	this.gr = gr;
	this.maxMoves = maxMoves;
    }

    public GameState play() {
	GameState gs = new GameState();
	for (int currentMove = 0; currentMove<maxMoves && gr.isWin(gs)==0; currentMove++) {
	    System.out.print(gs.toLongString(2)+"\n");
	    Move nextMove = ((gs.turn()==1) ? player_pos : player_neg).getNextMove(gs);
	    if (nextMove==null) {
		System.out.print(":::Quitting...\n");
		return gs;
	    }
	    if (!gr.isLegal(gs, nextMove)) {
		System.out.print(":::Illegal move "+nextMove.toString()+"\n");
		return gs;
	    }
	    else
		System.out.print("move"+currentMove+"> "+nextMove.toString()+"\n\n");
	    gs = gs.applyMove(nextMove);
	}
	System.out.print(gs.toLongString(2)+"\n");

	int w = gr.isWin(gs);
	if (w != 0)
	    System.out.print("Player \"" + (w > 0 ? "+" : "-") + "\" wins!\n");
	else
	    System.out.print("Nobody wins after " + maxMoves + " moves.\n");

	return gs;
    }

    static final int maxMoves_default = 1000; 

    private static void show_usage() {
	System.err.print("Usage: java Referee rules[+|-] player [maxMoves]\n" +
                         "  rules - alldiag, centerdiag, nodiag\n" +
                         "          optional +/- indicate whether you want to begin (default) or not\n" +
                         "          there is no space between rules and +/-\n" +
                         "  player - opt, opt90, random, stdin\n" +
                         "           choose an opponent that is optimal, optimal 90% of the time,\n" +
                         "           random, or one that takes keyboard input\n" +
                         "  maxMoves - maximum number of moves (default " + maxMoves_default + ")\n");
    }

    public static void main(String[] argv) throws IOException {
	Player player, myplayer;
	GameRules gr;
	Referee referee;
	int maxMoves = maxMoves_default;
	int myplayer_sign = 1;

	String game_id, player_id;

	try {
	    game_id = argv[0];
	    player_id = argv[1];
	    if (argv.length==3)
		maxMoves = Integer.parseInt(argv[2]);
	}
	catch (Exception e) {
	    show_usage();
	    return;
	}
	if (game_id.endsWith("-"))
	    myplayer_sign = -1;

	if (game_id.endsWith("+") || game_id.endsWith("-"))
	    game_id = game_id.substring(0,game_id.length()-1);
	
	try {
	    gr = new GameRules(game_id);
	}
	catch (IllegalArgumentException e) {
	    show_usage();
	    return;
	}
	if (player_id.equals("opt"))
	    player = new OptPlayer(game_id);
	else if (player_id.equals("opt90"))
	    player = new OptPlayer(game_id, 0.9);
	else if (player_id.equals("random"))
	    player = new OptPlayer(game_id, 0.0);
	else if (player_id.equals("stdin"))
	    player = new InteractPlayer(gr);
	else {
	    show_usage();
	    return;
	}
	myplayer = new MyPlayer(gr);
	
	if (myplayer_sign == +1)
	    referee = new Referee(myplayer, player, gr, maxMoves);
	else
	    referee = new Referee(player, myplayer, gr, maxMoves);
	
	GameState gs = referee.play();
    }
}
